package sun.util.locale.provider;

import java.util.Currency;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.Set;
import java.text.spi.NumberFormatProvider;

public class NumberFormatProviderImpl extends NumberFormatProvider implements AvailableLanguageTags
{
    private static final int NUMBERSTYLE = 0;
    private static final int CURRENCYSTYLE = 1;
    private static final int PERCENTSTYLE = 2;
    private static final int SCIENTIFICSTYLE = 3;
    private static final int INTEGERSTYLE = 4;
    private final LocaleProviderAdapter.Type type;
    private final Set<String> langtags;
    
    public NumberFormatProviderImpl(LocaleProviderAdapter.Type type, Set<String> langtags) {
        this.type = type;
        this.langtags = langtags;
    }
    
    @Override
    public Locale[] getAvailableLocales() {
        return LocaleProviderAdapter.forType(this.type).getAvailableLocales();
    }
    
    @Override
    public boolean isSupportedLocale(Locale locale) {
        return LocaleProviderAdapter.isSupportedLocale(locale, this.type, this.langtags);
    }
    
    @Override
    public NumberFormat getCurrencyInstance(Locale locale) {
        return this.getInstance(locale, 1);
    }
    
    @Override
    public NumberFormat getIntegerInstance(Locale locale) {
        return this.getInstance(locale, 4);
    }
    
    @Override
    public NumberFormat getNumberInstance(Locale locale) {
        return this.getInstance(locale, 0);
    }
    
    @Override
    public NumberFormat getPercentInstance(Locale locale) {
        return this.getInstance(locale, 2);
    }
    
    private NumberFormat getInstance(Locale locale, int n) {
        if (locale == null) {
            throw new NullPointerException();
        }
        String[] numberPatterns = LocaleProviderAdapter.forType(this.type).getLocaleResources(locale).getNumberPatterns();
        DecimalFormatSymbols instance = DecimalFormatSymbols.getInstance(locale);
        n = ((n == 4) ? 0 : n);
        DecimalFormat decimalFormat = new DecimalFormat(numberPatterns[n], instance);
        if (n == 4) {
            decimalFormat.setMaximumFractionDigits(0);
            decimalFormat.setDecimalSeparatorAlwaysShown(false);
            decimalFormat.setParseIntegerOnly(true);
        }
        else if (n == 1) {
            adjustForCurrencyDefaultFractionDigits(decimalFormat, instance);
        }
        return decimalFormat;
    }
    
    private static void adjustForCurrencyDefaultFractionDigits(DecimalFormat decimalFormat, DecimalFormatSymbols decimalFormatSymbols) {
        Currency currency = decimalFormatSymbols.getCurrency();
        if (currency == null) {
            try {
                currency = Currency.getInstance(decimalFormatSymbols.getInternationalCurrencySymbol());
            }
            catch (IllegalArgumentException ex) {}
        }
        if (currency != null) {
            int defaultFractionDigits = currency.getDefaultFractionDigits();
            if (defaultFractionDigits != -1) {
                int minimumFractionDigits = decimalFormat.getMinimumFractionDigits();
                if (minimumFractionDigits == decimalFormat.getMaximumFractionDigits()) {
                    decimalFormat.setMinimumFractionDigits(defaultFractionDigits);
                    decimalFormat.setMaximumFractionDigits(defaultFractionDigits);
                }
                else {
                    decimalFormat.setMinimumFractionDigits(Math.min(defaultFractionDigits, minimumFractionDigits));
                    decimalFormat.setMaximumFractionDigits(defaultFractionDigits);
                }
            }
        }
    }
    
    @Override
    public Set<String> getAvailableLanguageTags() {
        return this.langtags;
    }
}
